@@ -12,9 +12,10 @@ class ParamsMapper(object):
 
     _key_for_all_requests={
             ######################## for all services ########################
+            "files": {"type":list,"has_default":True,"default":[]},
             "file_path_info": {"type":DICTIONARY_LIST_TYPE,"has_default":True,"default":[]},
             "sheet":{"type":any,"has_default":True,"default":None},
-
+            "file_content_info":{"type":DICTIONARY_LIST_TYPE,"has_default":True,"default":[]}
 
             #########################******* for text analysis service *******########################
             "keywords_count": {"type":int,"has_default":True,"default":4},
@@ -29,6 +30,7 @@ class ParamsMapper(object):
             
             
             ########################******* for machine learning service ********########################
+            ########################*******                              ********########################
 
             #++++++++++++++++++++++++++++++ fit model ++++++++++++++++++++++++++++++#
             
@@ -72,6 +74,9 @@ class ParamsMapper(object):
             "clusters_column": {"type":any,"has_default":True,"default":None},
 
 
+            #++++++++++++++++++++++++++++++ For removing non wanted models ++++++++++++++++++++++++++++++#
+
+            "models_to_keep": {"type": list,"has_default":False},
 
             #++++++++++++++++++++++++++++++ Model Optmization(Clustering,Classification) ++++++++++++++++++++++++++++++#
 
@@ -81,7 +86,67 @@ class ParamsMapper(object):
             "stay_within": {"type":bool,"has_default":True,"default":False},
             "stay_below": {"type":bool,"has_default":True,"default":True},
             "stay_above": {"type":bool,"has_default":True,"default":True},
-            "stay_within_num": {"type":any,"has_default":True,"default":None}
+            "stay_within_num": {"type":any,"has_default":True,"default":None},
+            
+            
+            ########################*******  for Recommendation service  ********########################
+            ########################*******                              ********########################
+            
+            
+            #++++++++++++++++++++++++++++++ Association Rules ++++++++++++++++++++++++++++++#
+            "min_support":{"type":int,"has_default":True,"default":None},
+            "min_conf":{"type":int,"has_default":False},
+            "max_set_size":{"type":int,"has_default":True,"default":100},
+            "left_columns": {"type": list,"has_default":True,"default":[]},
+            "right_column": {"type": str,"has_default":True,"default":None},
+            "right_values": {"type": list,"has_default":True,"default":"all"},
+            "group_column": {"type": str,"has_default":True,"default":None},
+
+            ########################*******  for EDA service  ********########################
+            ########################*******                   ********########################
+            "algorithm_type":{"type":str,"has_default":False}
+            
+            #++++++++++++++++++++++++++++++ Explore Features for train ++++++++++++++++++++++++++++++#
+            "algorithm_type": {"type":str,"has_default":False},
+
+            #++++++++++++++++++++++++++++++ Explore column visualization data ++++++++++++++++++++++++++++++#
+            
+            "label":{"type":any,"has_default":True,"default":None}
+            
+            #++++++++++++++++++++++++++++++ features exploration data ++++++++++++++++++++++++++++++#
+            "statistic_type":{"type":str,"has_default":True,"default":"hist"}
+            
+            #++++++++++++++++++++++++++++++ outlier percentage ++++++++++++++++++++++++++++++#
+            
+            "outlier_percentage":{"type":float,"has_default":False}
+            "number_of_neighboors":{"type":float,"has_default":False}
+            
+            #++++++++++++++++++++++++++++++     For merge      ++++++++++++++++++++++++++++++#
+            "root_file": {"type": dict,"has_default":False},
+            "merge_files": {"type": DICTIONARY_LIST_TYPE,"has_default":False},
+            "link_columns": {"type": DICTIONARY_LIST_TYPE,"has_default":False},
+            "display_columns": {"type": DICTIONARY_LIST_TYPE,"has_default":False},
+            
+            #++++++++++++++++++++++++++++++     For join      ++++++++++++++++++++++++++++++#
+            "query_files":{"type":DICTIONARY_LIST_TYPE,"has_default":False}
+            "select_columns":{"type"list,"has_default":False}
+            
+            
+            
+            
+            #++++++++++++++++++++++++++++++ Custom Functions ++++++++++++++++++++++++++++++#
+            "column": {"type": "String"},
+            "template_guid": {"type": "DirectoryPath"},
+            "return_type": {"type": "String"},
+            "cutom_function_code": {"type": "String"},
+            "custom_function_keys": {"type": "List"},
+            "custom_functino_values": {"type": "List"}
+            
+            #++++++++++++++++++++++++++++++ Custom Functions ++++++++++++++++++++++++++++++#
+            
+            
+            
+            
     }
     @classmethod
 
@@ -131,7 +196,7 @@ class RequestParser():
 
         # self._passed_request.query_params.gitlist("file_path_info")
         foundKeys=np.intersect1d(list(self._request_dict.keys()),list(self._expected_keys.keys()))
-        self._request_dict=self._passed_request.data.copy()
+        self._request_dict=self._passed_request.data
         for key in foundKeys:
             properties=self._expected_keys[key]
             if properties["type"]==self.DICTIONARY_LIST_TYPE :
@@ -167,15 +232,19 @@ class RequestParser():
         for key,value in self._expected_keys.items():
             if key in requestkeys:
                 if value["type"]==self.DICTIONARY_LIST_TYPE:
-
-                        # example=value.get("example")
-                        # notFoundKeysInRequest.append(f"{key}:value like {example}")
-                    continue
+                    if isinstance(self._request_dict[key],dict):
+                        self._request_dict.update({key:[self._request_dict[key]]})
+                    
+                    elif not isinstance(self._request_dict[key],list):
+                        notFoundKeysInRequest.append(f"{key}:should be map key:value")
+                    else:
+                        for dictIntstance in self._request_dict[key]:
+                            if not isinstance(dictIntstance,dict):
+                                notFoundKeysInRequest.append(f"{key}:should be map key:value")
+                                break
 
                 elif value["type"]==dict:
-                    if isinstance(self._request_dict[key],dict):
-                        continue
-                    else:
+                    if not isinstance(self._request_dict[key],dict):
                         notFoundKeysInRequest.append(f"{key}:should be map key:value")
                         continue
                 elif value["type"]==list:
@@ -193,6 +262,11 @@ class RequestParser():
                 elif value["type"]==bool:
                     if not isinstance(self._request_dict[key],bool):
                         notFoundKeysInRequest.append(f"{key}:should be boolean True,False")
+                elif value["type"]==float or value["type"]==int:
+                    if not (isinstance(self._request_dict[key],float) or isinstance(self._request_dict[key],int)):
+                        notFoundKeysInRequest.append(f"{key}:should be boolean True,False")
+                    else:
+                        self._request_dict.update({key:value["type"](self._request_dict[key])})
                 elif value["type"]==any:
                     continue
                 else:
@@ -206,7 +280,7 @@ class RequestParser():
                 notFoundKeysInRequest.append(key)
 
         return True if len(notFoundKeysInRequest)==0 else False,self._request_dict if len(notFoundKeysInRequest)==0 else notFoundKeysInRequest
-
+    
     def ReadListFiles(self):
         passedFiles=self._passed_request.FILES.getlist("files",[])
         passedFilesData=[]
@@ -220,20 +294,21 @@ class RequestParser():
 
         if self._request_dict["file_path_info"] is not None and self._request_dict["file_path_info"] !=[] :
 
-            status,response=ServiceController.CallEurekaService(service_name=Services.FILE_SERVICE_NAME,service_url=Services.READ_DF_FILE_FROM_SERVICE,data={"file_path_info":self._request_dict["file_path_info"]},response_key=Services.DATA_KEY)
+            status,response=ServiceController.CallEurekaService(service_name=Services.FILE_SERVICE_NAME,service_url=Services.READ_DF_FILE,data={"file_path_info":self._request_dict["file_path_info"]},response_key=Services.DATA_KEY)
             if not status:
                 return response
-    
             passedFilesData.extend(response)
 
+        if len(passedFilesData)==0:
+            return {"message":"pass files","code":422}
+            
         for file in passedFilesData:
-            file.update({"template_guid":GetTemplateGuidFromPath["file_path"]})
-        return passedFiles
+            file.update({"template_guid":GetTemplateGuidFromPath(file["file_path"])})
+        return passedFilesData
 
     def ReadFile(self,file_path_info={}):
         
         passedFiles=None if file_path_info=={} else self._passed_request.FILES.get("files",None)
-        passedFilesData=[]
         if passedFiles is not None:
             status,response=ServiceController.CallEurekaService(Services.FILE_SERVICE_NAME,Services.UPLOAD_FILES_FROM_SERVICE,Services.UPLOAD_FILES_FROM_SERVICE,data={"files":base64.b64encode(self._passed_request.FILES.get("files").read()).decode()},response_key=Services.DATA_KEY)
             if not status:
@@ -244,6 +319,8 @@ class RequestParser():
 
 
         fileToRead=file_path_info["file_path_info"] if file_path_info!={} else self._request_dict["file_path_info"] 
+        if fileToRead==[]:
+            return {"message":"pass files","code":422}
         status,response=ServiceController.CallEurekaService(Services.FILE_SERVICE_NAME,Services.READ_DF_FILE,data={"file_path_info":fileToRead[0:1] if isinstance(fileToRead,list) else [fileToRead]},response_key=Services.DATA_KEY)
         if not status:
             return response
